class SakuraIOBaseException(ValueError):
    pass


class CommandError(SakuraIOBaseException):

    def __init__(self, status):
        self.status = status

    def __str__(self):
        return "Invalid response status '0x{0:02x}'".format(self.status)


class ParityError(SakuraIOBaseException):

    def __init__(self):
        pass

    def __str__(self):
        return "Invalid parity"



from .common import *
from .transmit import *
from .receive import *
from .file import *
from .operation import *

# Response
CMD_ERROR_NONE = 0x01
CMD_ERROR_PARITY = 0x02
CMD_ERROR_MISSING = 0x03
CMD_ERROR_INVALID_SYNTAX = 0x04
CMD_ERROR_RUNTIME = 0x05
CMD_ERROR_LOCKED = 0x06
CMD_ERROR_BUSY = 0x07


class CommandMixins(CommonMixins, TransmitMixins, ReceiveMixins, OperationMixins, FileMixins):
    pass



import datetime
import struct

from sakuraio.hardware.commands import CommandMixins, CMD_ERROR_NONE
from sakuraio.hardware.exceptions import CommandError, ParityError

SAKURAIO_SLAVE_ADDR = 0x4f


def calc_parity(values):
    parity = 0x00
    for value in values:
        parity ^= value
    return parity


class SakuraIOBase(CommandMixins):

    def start(self, write=True):
        return

    def end(self):
        return

    def send_byte(self, value):
        raise NotImplementedError()

    def recv_byte(self):
        raise NotImplementedError()

    def execute_command(self, cmd, request=[], as_bytes=False):
        response = []

        request = [cmd, len(request)] + request
        request.append(calc_parity(request))

        try:
            # Request
            self.start(True)
            for value in request:
                self.send_byte(value)

            # Response
            self.start(False)
            status = self.recv_byte()
            if status != CMD_ERROR_NONE:
                raise CommandError(status)

            length = self.recv_byte()
            response = []
            for i in range(length):
                response.append(self.recv_byte())
            parity = self.recv_byte()

            if parity != calc_parity([status, length] + response):
                raise ParityError()

        except:
            self.end()
            raise

        self.end()

        if as_bytes:
            return struct.pack("B" * len(response), *response)
        return response



import struct
import datetime

from .utils import pack

# Transmit
CMD_TX_ENQUEUE = 0x20
CMD_TX_SENDIMMED = 0x21
CMD_TX_LENGTH = 0x22
CMD_TX_CLEAR = 0x23
CMD_TX_SEND = 0x24
CMD_TX_STAT = 0x25


class TransmitMixins(object):

    def enqueue_tx_raw(self, channel, type, values, offset=0):
        """Enqueue channel data by raw values.

        :param int channel:
            Channel number of data. Must be 0 to 127.

        :param string type:
            Type of data. Possible values ``"i"``, ``"I"``, ``"l"``, ``"L"``, ``"f"``, ``"d"`` or ``"b"``.

        :param list values:
            List of int values to enqueue.

        :params int offset:
            Time offset in ms. Default ``0``. It must be less than or equal ``0``.
        """

        values += [0x00] * 8

        if type not in ["i", "I", "l", "L", "f", "d", "b"]:
            raise ValueError("Invalid type '{0}'".format(type))

        request = [channel, ord(type)] + values[:8]
        if offset != 0:
            request += pack("<q", offset)

        self.execute_command(CMD_TX_ENQUEUE, request)

    def send_immediate_raw(self, channel, type, data):
        """Send channel data immediately by raw values.

        :param int channel:
            Channel number of data. Must be 0 to 127.

        :param string type:
            Type of data. Possible values ``"i"``, ``"I"``, ``"l"``, ``"L"``, ``"f"``, ``"d"`` or ``"b"``.

        :param list values:
            List of int values to send.

        """

        data += [0x00] * 8

        if type not in ["i", "I", "l", "L", "f", "d", "b"]:
            raise ValueError("Invalid type '{0}'".format(type))

        request = [channel, ord(type)] + data[:8]
        self.execute_command(CMD_TX_SENDIMMED, request)

    def get_tx_queue_length(self):
        """Get available and queued length of tramsmit queue.

        :return: Size of available and queued data.
        :rtype: dict
        """

        response = self.execute_command(CMD_TX_LENGTH)
        return {
            "available": response[0],
            "queued": response[1],
        }

    def clear_tx(self):
        """Clear transmit queue."""

        self.execute_command(CMD_TX_CLEAR)

    def send(self):
        """Send data in transmit queue."""

        self.execute_command(CMD_TX_SEND)

    def get_tx_status(self):
        """Get status of send

        :return: Status of send.
        :rtype: dict
        """

        response = self.execute_command(CMD_TX_STAT)
        return {
            "queue": response[0],
            "immediate": response[1],
        }



import struct
import datetime

# File Download
CMD_START_FILE_DOWNLOAD = 0x40
CMD_GET_FILE_METADATA = 0x41
CMD_GET_FILE_DOWNLOAD_STATUS = 0x42
CMD_CANCEL_FILE_DOWNLOAD = 0x43
CMD_GET_FILE_DATA = 0x44


class FileMixins(object):

    def start_file_download(self, fileid):
        """Start file download

        :param list fileid:
            FileID. List length must be 1, the Value fileid[0] must be 1 to 5.

        """

        fileid += [0x00] * 2
        self.execute_command(CMD_START_FILE_DOWNLOAD, fileid[:2])

    def get_file_metadata(self):
        """Get file metadata

        :return: Dict of file metadata (status, filesize, timestamp, checksum).
        :rtype: dict
        """

        response = self.execute_command(CMD_GET_FILE_METADATA)
        return {
            "status": response[0],
            "size": struct.unpack("<i", struct.pack("4B", *response[1:5]))[0],
            "timestamp": struct.unpack("<q", struct.pack("8B", *response[5:13]))[0],
            "checksum": response[13:17],
        }

    def get_file_download_status(self):
        """Get file download status

        :return: Dict of download status and received datasize.
        :rtype: dict
        """

        response = self.execute_command(CMD_GET_FILE_DOWNLOAD_STATUS)
        return {
            "status": response[0],
            "size": struct.unpack("<i", struct.pack("4B", *response[1:5]))[0],
        }

    def cancel_file_download(self):
        """Cancel file download"""

        self.execute_command(CMD_CANCEL_FILE_DOWNLOAD)

    def get_file_data(self, rsize):
        """Get file data

        :return: Dict of download file body.abs
        :rtype: dict
        """

        response = self.execute_command(CMD_GET_FILE_DATA, rsize[0:1])
        return {
            "data": response[0:]
        }



import struct
import datetime

# Receive
CMD_RX_DEQUEUE = 0x30
CMD_RX_PEEK = 0x31
CMD_RX_LENGTH = 0x32
CMD_RX_CLEAR = 0x33


class ReceiveMixins(object):

    def dequeue_rx_raw(self):
        """Dequeue received data

        :return: Dict of received data.
        :rtype: dict
        """

        response = self.execute_command(CMD_RX_DEQUEUE)
        return {
            "channel": response[0],
            "type": chr(response[1]),
            "data": response[2:10],
            "offset": struct.unpack("<q", struct.pack("8B", *response[10:18]))[0],
        }

    def peek_rx_raw(self):
        """Peek received data

        :return: Dict of received data.
        :rtype: dict
        """

        response = self.execute_command(CMD_RX_PEEK)
        return {
            "channel": response[0],
            "type": chr(response[1]),
            "data": response[2:10],
            "offset": struct.unpack("<q", struct.pack("8B", *response[10:18]))[0],
        }

    def get_rx_queue_length(self):
        """Get available and queued length of receive queue.

        :return: Size of available and queued data.
        :rtype: dict
        """

        response = self.execute_command(CMD_RX_LENGTH)
        return {
            "available": response[0],
            "queued": response[1],
        }

    def clear_rx(self):
        """Clear receive queue."""

        self.execute_command(CMD_RX_CLEAR)



from sakuraio.hardware.base import SakuraIOBase


class DummySakuraIO(SakuraIOBase):

    def start(self, write=True):
        pass

    def end(self):
        pass

    def initial(self, response):
        self.return_values = response
        self.values = []

    def send_byte(self, value):
        self.values.append(value)

    def recv_byte(self):
        value = self.return_values[0]
        self.return_values = self.return_values[1:]
        return value



import struct
import datetime

# Common
CMD_GET_CONNECTION_STATUS = 0x01
CMD_GET_SIGNAL_QUALITY = 0x02
CMD_GET_DATETIME = 0x03
CMD_ECHO_BACK = 0x0f

CONNECTION_ERROR_NONE = 0x00
CONNECTION_ERROR_OUT_OF_SERVICE = 0x01
CONNECTION_ERROR_CONNECTION = 0x02
CONNECTION_ERROR_DISCONNECTED = 0x03


class CommonMixins(object):

    def get_connection_status(self):
        """Get connection status

        :return: Status. Please see the datasheet.
        :rtype: int
        """
        response = self.execute_command(CMD_GET_CONNECTION_STATUS)
        return response[0]

    def get_is_online(self):
        """Get online

        :return: Weather or not the module is online.
        :rtype: bool
        """

        status = self.get_connection_status()
        return (status & 0x80) == 0x80

    def get_connection_error(self):
        """Get connection error

        :return: Status. Possible values:
            :const:`CONNECTION_ERROR_NONE`, :const:`CONNECTION_ERROR_OUT_OF_SERVICE`,
            :const:`CONNECTION_ERROR_CONNECTION`, :const:`CONNECTION_ERROR_DISCONNECTED`
        :rtype: int
        """
        status = self.get_connection_status()
        return (status & 0x7f)

    def get_signal_quality(self):
        """Get signal quality

        :return: Signal quality. ``0``: out of service. ``5``: most strong.
        :rtype: int
        """
        response = self.execute_command(CMD_GET_SIGNAL_QUALITY)
        return response[0]

    def get_datetime(self):
        """Get current datetime

        :return: Current datetime.
        :rtype: datetime.datetime
        """

        response = self.execute_command(CMD_GET_DATETIME, as_bytes=True)
        unixtime, = struct.unpack("<Q", response)
        return datetime.datetime.utcfromtimestamp(unixtime / 1000.0)

    def echoback(self, values):
        """Test echoback MCU <-> Communication Module

        :param list values:
            List of int values to send.

        :return: Values echoed. It must equals ``values`` param.
        :rtype: list
        """
        response = self.execute_command(CMD_ECHO_BACK, values)
        return response



import struct


def pack(fmt, *args):
    """pack() is wrapper of struct.pack
    """

    value = struct.pack(fmt, *args)
    result = []

    if isinstance(value, str):
        # For Python2
        for v in value:
            result.append(ord(v))

        return result
    elif isinstance(value, bytes):
        # For Python3
        return value
    else:
        TypeError()



from sakuraio.hardware.base import SAKURAIO_SLAVE_ADDR, SakuraIOBase


class SakuraIOSMBus(SakuraIOBase):

    def __init__(self):
        import smbus
        self.bus = smbus.SMBus(1)

    def start(self, write=True):
        if write:
            self.request = []
            self.response = []
        else:
            if self.request:
                self.bus.write_i2c_block_data(SAKURAIO_SLAVE_ADDR, self.request[0], self.request[1:])
            self.response = self.bus.read_i2c_block_data(SAKURAIO_SLAVE_ADDR, 32)

    def send_byte(self, value):
        self.request.append(value)

    def recv_byte(self):
        value = 0x00
        if self.response:
            value = self.response.pop(0)
        return value


class SakuraIOGPIO(SakuraIOBase):

    def __init__(self, miso=9, mosi=10, clk=11, cs=8):
        from RPi import GPIO
        self.GPIO = GPIO
        self.miso = miso
        self.mosi = mosi
        self.clk = clk
        self.cs = cs
        GPIO.setmode(GPIO.BCM)
        GPIO.setup(self.miso, GPIO.IN)
        GPIO.setup(self.mosi, GPIO.OUT)
        GPIO.setup(self.clk, GPIO.OUT)
        GPIO.setup(self.cs, GPIO.OUT)

        self.GPIO.output(self.cs, self.GPIO.HIGH)
        self.GPIO.output(self.clk, self.GPIO.LOW)

    def start(self, write=True):
        self.GPIO.output(self.cs, self.GPIO.LOW)

    def end(self):
        self.GPIO.output(self.cs, self.GPIO.HIGH)

    def send_byte(self, value):
        ret = 0x00
        for bit in range(8):

            if value & 0x80:
                self.GPIO.output(self.mosi, self.GPIO.HIGH)
            else:
                self.GPIO.output(self.mosi, self.GPIO.LOW)

            self.GPIO.output(self.clk, self.GPIO.HIGH)

            ret <<= 1
            if self.GPIO.input(self.miso):
                ret |= 0x01

            self.GPIO.output(self.clk, self.GPIO.LOW)

            value <<= 1

        return ret

    def recv_byte(self):
        return self.send_byte(0x00)


class SakuraIOSPI(SakuraIOBase):
    def __init__(self):
        raise NotImplementedError()


class SakuraIOSerial(SakuraIOBase):

    def __init__(self, port, baudrate=115200):
        import serial
        self.serial = serial.Serial(port, baudrate, timeout=1)

    def __del__(self):
        self.serial.close()

    def readline(self):
        line = ""
        while True:
            c = self.serial.read().decode("ascii", "ignore")
            if not c:
                return line

            if c not in ("\r", "\n"):
                line += c
            elif len(line) > 0:
                return line

    def start(self, write=True):
        if write:
            self.request = "AT*CMD="
            self.response = []
        else:
            self.serial.write((self.request + "\n").encode("ascii", "ignore"))
            while True:
                line = self.readline()
                if not line:
                    return
                if line.startswith("*CMD:"):
                    response_hex = line[5:]
                    while response_hex:
                        self.response.append(int("0x" + response_hex[:2], 0))
                        response_hex = response_hex[2:]
                    return

    def send_byte(self, value):
        self.request += "{0:02X}".format(value)

    def recv_byte(self):
        value = 0x00
        if self.response:
            value = self.response.pop(0)
        return value



import struct
import datetime

# Operation
CMD_GET_PRODUCT_ID = 0xA0
CMD_GET_UNIQUE_ID = 0xA1
CMD_GET_FIRMWARE_VERSION = 0xA2
CMD_UNLOCK = 0xA8
CMD_UPDATE_FIRMWARE = 0xA9
CMD_GET_FIRMWARE_UPDATE_STATUS = 0xAA
CMD_SOFTWARE_RESET = 0xAF

UNLOCK_MAGIC_NUMBERS = [0x53, 0x6B, 0x72, 0x61]


PRODUCT_ID_SCM_LTE_BETA = 0x01
PRODUCT_ID_SCM_LTE_01 = 0x02

PRODUCT_ID_MAP = {
    PRODUCT_ID_SCM_LTE_BETA: "SCM-LTE-BETA",
    PRODUCT_ID_SCM_LTE_01: "SCM-LTE-01",
}


class OperationMixins(object):

    def get_product_id(self):
        """Get product id

        :return: Product ID. Possible values:
            :const:`PRODUCT_ID_SCM_LTE_BETA`, :const:`PRODUCT_ID_SCM_LTE_01`
        :rtype: int
        """

        response = self.execute_command(CMD_GET_PRODUCT_ID, as_bytes=True)
        product_id = struct.unpack("<H", response)[0]
        return product_id

    def get_product_name(self):
        """Get product name

        :return: Product name. Possible values: ``"SCM-LTE-BETA"``, ``"SCM-LTE-01"``.
        :rtype: str
        """

        product_id = self.get_product_id()
        return PRODUCT_ID_MAP.get(product_id, "{0:04X}".format(product_id))

    def get_unique_id(self):
        """Get unique id

        :return: Unique ID. For example `"16X0000001"``.
        :rtype: str
        """

        return self.execute_command(CMD_GET_UNIQUE_ID, as_bytes=True).decode("ascii")

    def get_firmware_version(self):
        """Get firmware version

        :return: Firmware version. For example `"v1.1.2-170223-7e6ce64"``.
        :rtype: str
        """
        return self.execute_command(CMD_GET_FIRMWARE_VERSION, as_bytes=True).decode("ascii")

    def unlock(self):
        """Unlock critical command"""

        self.execute_command(CMD_UNLOCK, UNLOCK_MAGIC_NUMBERS)

    def update_firmware(self):
        """Request to update firmware"""
        self.execute_command(CMD_UPDATE_FIRMWARE)

    def get_firmware_update_status(self):
        """Get firmware update status

        :return: Status.
        :rtype: dict
        """

        response = self.execute_command(CMD_GET_FIRMWARE_UPDATE_STATUS)[0]
        inprogress = (response & 0x80) == 0x80
        return {
            "inprogress": inprogress,
            "error": response & 0x7f,
        }

    def reset(self):
        """Request software reset"""

        self.execute_command(CMD_SOFTWARE_RESET)



#!/usr/bin/env python

from distutils.core import setup

setup(
    name='SakuraIO',
    version='0.1.4',
    description='Python SakuraIO Library',
    author='chibiegg',
    author_email='chibiegg@chibiegg.net',
    url='https://github.com/sakuraio/python-sakuraio/',
    packages=['sakuraio', 'sakuraio.hardware', 'sakuraio.hardware.commands', 'sakuraio.hardware.rpi'],
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 4 - Beta',
        # Indicate who your project is intended for
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: System :: Hardware',
        'Topic :: Utilities',
        'Topic :: Communications',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)
